@@ -9,9 +9,10 @@ use crate::unicode::*;
 
 use serde::ser::{Serialize, SerializeMap, Serializer};
 
+use smallvec::SmallVec;
 use std::ptr::NonNull;
 
-pub struct DataclassGenericSerializer {
+pub struct Dataclass {
     ptr: *mut pyo3::ffi::PyObject,
     opts: Opt,
     default_calls: u8,
@@ -19,7 +20,7 @@ pub struct DataclassGenericSerializer {
     default: Option<NonNull<pyo3::ffi::PyObject>>,
 }
 
-impl DataclassGenericSerializer {
+impl Dataclass {
     pub fn new(
         ptr: *mut pyo3::ffi::PyObject,
         opts: Opt,
@@ -27,7 +28,7 @@ impl DataclassGenericSerializer {
         recursion: u8,
         default: Option<NonNull<pyo3::ffi::PyObject>>,
     ) -> Self {
-        DataclassGenericSerializer {
+        Dataclass {
             ptr: ptr,
             opts: opts,
             default_calls: default_calls,
@@ -37,17 +38,15 @@ impl DataclassGenericSerializer {
     }
 }
 
-impl Serialize for DataclassGenericSerializer {
+impl Serialize for Dataclass {
     #[inline(never)]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
     {
-        let dict = ffi!(PyObject_GetAttr(self.ptr, DICT_STR));
         let ob_type = ob_type!(self.ptr);
-        if unlikely!(dict.is_null()) {
-            ffi!(PyErr_Clear());
-            DataclassFallbackSerializer::new(
+        if pydict_contains!(ob_type, SLOTS_STR) {
+            DataclassFields::new(
                 self.ptr,
                 self.opts,
                 self.default_calls,
@@ -55,33 +54,33 @@ impl Serialize for DataclassGenericSerializer {
                 self.default,
             )
             .serialize(serializer)
-        } else if pydict_contains!(ob_type, SLOTS_STR) {
-            let ret = DataclassFallbackSerializer::new(
+        } else {
+            match AttributeDict::new(
                 self.ptr,
                 self.opts,
                 self.default_calls,
                 self.recursion,
                 self.default,
-            )
-            .serialize(serializer);
-            ffi!(Py_DECREF(dict));
-            ret
-        } else {
-            let ret = DataclassFastSerializer::new(
-                dict,
-                self.opts,
-                self.default_calls,
-                self.recursion,
-                self.default,
-            )
-            .serialize(serializer);
-            ffi!(Py_DECREF(dict));
-            ret
+            ) {
+                Ok(val) => val.serialize(serializer),
+                Err(AttributeDictError::DictMissing) => DataclassFields::new(
+                    self.ptr,
+                    self.opts,
+                    self.default_calls,
+                    self.recursion,
+                    self.default,
+                )
+                .serialize(serializer),
+            }
         }
     }
 }
 
-pub struct DataclassFastSerializer {
+pub enum AttributeDictError {
+    DictMissing,
+}
+
+pub struct AttributeDict {
     ptr: *mut pyo3::ffi::PyObject,
     opts: Opt,
     default_calls: u8,
@@ -89,34 +88,46 @@ pub struct DataclassFastSerializer {
     default: Option<NonNull<pyo3::ffi::PyObject>>,
 }
 
-impl DataclassFastSerializer {
+impl AttributeDict {
     pub fn new(
         ptr: *mut pyo3::ffi::PyObject,
         opts: Opt,
         default_calls: u8,
         recursion: u8,
         default: Option<NonNull<pyo3::ffi::PyObject>>,
-    ) -> Self {
-        DataclassFastSerializer {
-            ptr: ptr,
+    ) -> Result<Self, AttributeDictError> {
+        let dict = ffi!(PyObject_GetAttr(ptr, DICT_STR));
+        if unlikely!(dict.is_null()) {
+            ffi!(PyErr_Clear());
+            return Err(AttributeDictError::DictMissing);
+        }
+        Ok(AttributeDict {
+            ptr: dict,
             opts: opts,
             default_calls: default_calls,
             recursion: recursion,
             default: default,
-        }
+        })
     }
 }
 
-impl Serialize for DataclassFastSerializer {
+impl Drop for AttributeDict {
+    fn drop(&mut self) {
+        ffi!(Py_DECREF(self.ptr));
+    }
+}
+
+impl Serialize for AttributeDict {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
     {
-        let len = ffi!(Py_SIZE(self.ptr));
+        let len = ffi!(Py_SIZE(self.ptr)) as usize;
         if unlikely!(len == 0) {
             return serializer.serialize_map(Some(0)).unwrap().end();
         }
-        let mut map = serializer.serialize_map(None).unwrap();
+        let mut items: SmallVec<[(&str, *mut pyo3::ffi::PyObject); 8]> =
+            SmallVec::with_capacity(len);
         for (key, value) in PyDictIter::from_pyobject(self.ptr) {
             if unlikely!(!is_type!(ob_type!(key.as_ptr()), STR_TYPE)) {
                 err!(KEY_MUST_BE_STR)
@@ -129,22 +140,26 @@ impl Serialize for DataclassFastSerializer {
             if unlikely!(key_as_str.as_bytes()[0] == b'_') {
                 continue;
             }
+            items.push((key_as_str, value.as_ptr()));
+        }
 
-            let pyvalue = PyObjectSerializer::new(
-                value.as_ptr(),
+        let mut map = serializer.serialize_map(Some(items.len())).unwrap();
+        for (key, value) in items.iter() {
+            let pyvalue = PyObject::new(
+                *value,
                 self.opts,
                 self.default_calls,
                 self.recursion + 1,
                 self.default,
             );
-            map.serialize_key(key_as_str).unwrap();
+            map.serialize_key(key).unwrap();
             map.serialize_value(&pyvalue)?;
         }
         map.end()
     }
 }
 
-pub struct DataclassFallbackSerializer {
+pub struct DataclassFields {
     ptr: *mut pyo3::ffi::PyObject,
     opts: Opt,
     default_calls: u8,
@@ -152,7 +167,7 @@ pub struct DataclassFallbackSerializer {
     default: Option<NonNull<pyo3::ffi::PyObject>>,
 }
 
-impl DataclassFallbackSerializer {
+impl DataclassFields {
     pub fn new(
         ptr: *mut pyo3::ffi::PyObject,
         opts: Opt,
@@ -160,7 +175,7 @@ impl DataclassFallbackSerializer {
         recursion: u8,
         default: Option<NonNull<pyo3::ffi::PyObject>>,
     ) -> Self {
-        DataclassFallbackSerializer {
+        DataclassFields {
             ptr: ptr,
             opts: opts,
             default_calls: default_calls,
@@ -170,18 +185,19 @@ impl DataclassFallbackSerializer {
     }
 }
 
-impl Serialize for DataclassFallbackSerializer {
+impl Serialize for DataclassFields {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
     {
         let fields = ffi!(PyObject_GetAttr(self.ptr, DATACLASS_FIELDS_STR));
         ffi!(Py_DECREF(fields));
-        let len = ffi!(Py_SIZE(fields));
+        let len = ffi!(Py_SIZE(fields)) as usize;
         if unlikely!(len == 0) {
             return serializer.serialize_map(Some(0)).unwrap().end();
         }
-        let mut map = serializer.serialize_map(None).unwrap();
+        let mut items: SmallVec<[(&str, *mut pyo3::ffi::PyObject); 8]> =
+            SmallVec::with_capacity(len);
         for (attr, field) in PyDictIter::from_pyobject(fields) {
             let field_type = ffi!(PyObject_GetAttr(field.as_ptr(), FIELD_TYPE_STR));
             ffi!(Py_DECREF(field_type));
@@ -199,14 +215,19 @@ impl Serialize for DataclassFallbackSerializer {
 
             let value = ffi!(PyObject_GetAttr(self.ptr, attr.as_ptr()));
             ffi!(Py_DECREF(value));
-            let pyvalue = PyObjectSerializer::new(
-                value,
+            items.push((key_as_str, value));
+        }
+
+        let mut map = serializer.serialize_map(Some(items.len())).unwrap();
+        for (key, value) in items.iter() {
+            let pyvalue = PyObject::new(
+                *value,
                 self.opts,
                 self.default_calls,
                 self.recursion + 1,
                 self.default,
             );
-            map.serialize_key(key_as_str).unwrap();
+            map.serialize_key(key).unwrap();
             map.serialize_value(&pyvalue)?
         }
         map.end()
