@@ -6,7 +6,7 @@ use crate::serialize::serializer::*;
 use serde::ser::{Serialize, SerializeSeq, Serializer};
 use std::ptr::NonNull;
 
-pub struct ListSerializer {
+pub struct List {
     ptr: *mut pyo3::ffi::PyObject,
     opts: Opt,
     default_calls: u8,
@@ -14,7 +14,7 @@ pub struct ListSerializer {
     default: Option<NonNull<pyo3::ffi::PyObject>>,
 }
 
-impl ListSerializer {
+impl List {
     pub fn new(
         ptr: *mut pyo3::ffi::PyObject,
         opts: Opt,
@@ -22,7 +22,7 @@ impl ListSerializer {
         recursion: u8,
         default: Option<NonNull<pyo3::ffi::PyObject>>,
     ) -> Self {
-        ListSerializer {
+        List {
             ptr: ptr,
             opts: opts,
             default_calls: default_calls,
@@ -32,19 +32,17 @@ impl ListSerializer {
     }
 }
 
-impl Serialize for ListSerializer {
+impl Serialize for List {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
     {
         let len = ffi!(PyList_GET_SIZE(self.ptr)) as usize;
         let mut seq = serializer.serialize_seq(Some(len)).unwrap();
-        let slice: &[*mut pyo3::ffi::PyObject] = unsafe {
-            std::slice::from_raw_parts((*(self.ptr as *mut pyo3::ffi::PyListObject)).ob_item, len)
-        };
-        for &each in slice {
-            let value = PyObjectSerializer::new(
-                each,
+        for i in 0..len {
+            let item = ffi!(PyList_GET_ITEM(self.ptr, i as isize));
+            let value = PyObject::new(
+                item,
                 self.opts,
                 self.default_calls,
                 self.recursion + 1,
