@@ -17,7 +17,7 @@ use crate::serialize::tuple::*;
 use crate::serialize::uuid::*;
 use crate::serialize::writer::*;
 use crate::typeref::*;
-use serde::ser::{Serialize, SerializeSeq, Serializer};
+use serde::ser::{Serialize, Serializer};
 use std::ptr::NonNull;
 
 pub const RECURSION_LIMIT: u8 = 255;
@@ -28,7 +28,7 @@ pub fn serialize(
     opts: Opt,
 ) -> Result<NonNull<pyo3::ffi::PyObject>, String> {
     let mut buf = BytesWriter::default();
-    let obj = PyObjectSerializer::new(ptr, opts, 0, 0, default);
+    let obj = PyObject::new(ptr, opts, 0, 0, default);
     let mut ser = rmp_serde::Serializer::new(&mut buf);
     let res = obj.serialize(&mut ser);
     match res {
@@ -155,7 +155,7 @@ pub fn pyobject_to_obtype_unlikely(obj: *mut pyo3::ffi::PyObject, opts: Opt) ->
     }
 }
 
-pub struct PyObjectSerializer {
+pub struct PyObject {
     ptr: *mut pyo3::ffi::PyObject,
     obtype: ObType,
     opts: Opt,
@@ -164,7 +164,7 @@ pub struct PyObjectSerializer {
     default: Option<NonNull<pyo3::ffi::PyObject>>,
 }
 
-impl PyObjectSerializer {
+impl PyObject {
     pub fn new(
         ptr: *mut pyo3::ffi::PyObject,
         opts: Opt,
@@ -172,7 +172,7 @@ impl PyObjectSerializer {
         recursion: u8,
         default: Option<NonNull<pyo3::ffi::PyObject>>,
     ) -> Self {
-        PyObjectSerializer {
+        PyObject {
             ptr: ptr,
             obtype: pyobject_to_obtype(ptr, opts),
             opts: opts,
@@ -183,31 +183,34 @@ impl PyObjectSerializer {
     }
 }
 
-impl Serialize for PyObjectSerializer {
+impl Serialize for PyObject {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
     {
         match self.obtype {
-            ObType::Str => StrSerializer::new(self.ptr).serialize(serializer),
-            ObType::Bytes => BytesSerializer::new(self.ptr).serialize(serializer),
-            ObType::StrSubclass => StrSubclassSerializer::new(self.ptr).serialize(serializer),
-            ObType::Int => IntSerializer::new(self.ptr).serialize(serializer),
+            ObType::Str => Str::new(self.ptr).serialize(serializer),
+            ObType::Bytes => Bytes::new(self.ptr).serialize(serializer),
+            ObType::StrSubclass => StrSubclass::new(self.ptr).serialize(serializer),
+            ObType::Int => Int::new(self.ptr).serialize(serializer),
             ObType::None => serializer.serialize_unit(),
             ObType::Float => serializer.serialize_f64(ffi!(PyFloat_AS_DOUBLE(self.ptr))),
             ObType::Bool => serializer.serialize_bool(unsafe { self.ptr == TRUE }),
-            ObType::Datetime => DateTime::new(self.ptr, self.opts).serialize(serializer),
+            ObType::Datetime => match DateTime::new(self.ptr, self.opts) {
+                Ok(val) => val.serialize(serializer),
+                Err(err) => err!(err),
+            },
             ObType::Date => Date::new(self.ptr).serialize(serializer),
             ObType::Time => match Time::new(self.ptr, self.opts) {
                 Ok(val) => val.serialize(serializer),
-                Err(TimeError::HasTimezone) => err!(TIME_HAS_TZINFO),
+                Err(err) => err!(err),
             },
             ObType::Uuid => UUID::new(self.ptr).serialize(serializer),
             ObType::Dict => {
                 if unlikely!(self.recursion == RECURSION_LIMIT) {
                     err!(RECURSION_LIMIT_REACHED)
                 }
-                DictGenericSerializer::new(
+                Dict::new(
                     self.ptr,
                     self.opts,
                     self.default_calls,
@@ -220,32 +223,33 @@ impl Serialize for PyObjectSerializer {
                 if unlikely!(self.recursion == RECURSION_LIMIT) {
                     err!(RECURSION_LIMIT_REACHED)
                 }
-                if unlikely!(ffi!(PyList_GET_SIZE(self.ptr)) == 0) {
-                    serializer.serialize_seq(Some(0)).unwrap().end()
-                } else {
-                    ListSerializer::new(
-                        self.ptr,
-                        self.opts,
-                        self.default_calls,
-                        self.recursion,
-                        self.default,
-                    )
-                    .serialize(serializer)
+                List::new(
+                    self.ptr,
+                    self.opts,
+                    self.default_calls,
+                    self.recursion,
+                    self.default,
+                )
+                .serialize(serializer)
+            }
+            ObType::Tuple => {
+                if unlikely!(self.recursion == RECURSION_LIMIT) {
+                    err!(RECURSION_LIMIT_REACHED)
                 }
+                Tuple::new(
+                    self.ptr,
+                    self.opts,
+                    self.default_calls,
+                    self.recursion,
+                    self.default,
+                )
+                .serialize(serializer)
             }
-            ObType::Tuple => TupleSerializer::new(
-                self.ptr,
-                self.opts,
-                self.default_calls,
-                self.recursion,
-                self.default,
-            )
-            .serialize(serializer),
             ObType::Dataclass => {
                 if unlikely!(self.recursion == RECURSION_LIMIT) {
                     err!(RECURSION_LIMIT_REACHED)
                 }
-                DataclassGenericSerializer::new(
+                Dataclass::new(
                     self.ptr,
                     self.opts,
                     self.default_calls,
@@ -258,25 +262,21 @@ impl Serialize for PyObjectSerializer {
                 if unlikely!(self.recursion == RECURSION_LIMIT) {
                     err!(RECURSION_LIMIT_REACHED)
                 }
-                let dict = ffi!(PyObject_GetAttr(self.ptr, DICT_STR));
-                if unlikely!(dict.is_null()) {
-                    err!(PYDANTIC_MUST_HAVE_DICT)
-                } else {
-                    ffi!(Py_DECREF(dict));
-                    DataclassFastSerializer::new(
-                        dict,
-                        self.opts,
-                        self.default_calls,
-                        self.recursion,
-                        self.default,
-                    )
-                    .serialize(serializer)
+                match AttributeDict::new(
+                    self.ptr,
+                    self.opts,
+                    self.default_calls,
+                    self.recursion,
+                    self.default,
+                ) {
+                    Ok(val) => val.serialize(serializer),
+                    Err(AttributeDictError::DictMissing) => err!(PYDANTIC_MUST_HAVE_DICT),
                 }
             }
             ObType::Enum => {
                 let value = ffi!(PyObject_GetAttr(self.ptr, VALUE_STR));
                 ffi!(Py_DECREF(value));
-                PyObjectSerializer::new(
+                PyObject::new(
                     value,
                     self.opts,
                     self.default_calls,
@@ -285,14 +285,14 @@ impl Serialize for PyObjectSerializer {
                 )
                 .serialize(serializer)
             }
-            ObType::NumpyArray => match NumpyArray::new(self.ptr) {
+            ObType::NumpyArray => match NumpyArray::new(self.ptr, self.opts) {
                 Ok(val) => val.serialize(serializer),
                 Err(PyArrayError::Malformed) => err!("numpy array is malformed"),
                 Err(PyArrayError::NotContiguous) | Err(PyArrayError::UnsupportedDataType) => {
                     if self.default.is_none() {
                         err!("numpy array is not C contiguous; use ndarray.tolist() in default")
                     } else {
-                        DefaultSerializer::new(
+                        Default::new(
                             self.ptr,
                             self.opts,
                             self.default_calls,
@@ -303,9 +303,9 @@ impl Serialize for PyObjectSerializer {
                     }
                 }
             },
-            ObType::NumpyScalar => NumpyScalar::new(self.ptr).serialize(serializer),
-            ObType::Ext => ExtSerializer::new(self.ptr).serialize(serializer),
-            ObType::Unknown => DefaultSerializer::new(
+            ObType::NumpyScalar => NumpyScalar::new(self.ptr, self.opts).serialize(serializer),
+            ObType::Ext => Ext::new(self.ptr).serialize(serializer),
+            ObType::Unknown => Default::new(
                 self.ptr,
                 self.opts,
                 self.default_calls,
