@@ -13,6 +13,7 @@ pub struct NumpyTypes {
     pub array: *mut PyTypeObject,
     pub float64: *mut PyTypeObject,
     pub float32: *mut PyTypeObject,
+    pub float16: *mut PyTypeObject,
     pub int64: *mut PyTypeObject,
     pub int32: *mut PyTypeObject,
     pub int16: *mut PyTypeObject,
@@ -22,6 +23,7 @@ pub struct NumpyTypes {
     pub uint16: *mut PyTypeObject,
     pub uint8: *mut PyTypeObject,
     pub bool_: *mut PyTypeObject,
+    pub datetime64: *mut PyTypeObject,
 }
 
 pub static mut DEFAULT: *mut PyObject = null_mut();
@@ -65,6 +67,8 @@ pub static mut PYDANTIC_FIELDS_STR: *mut PyObject = null_mut();
 pub static mut PYDANTIC2_FIELDS_STR: *mut PyObject = null_mut();
 pub static mut FIELD_TYPE_STR: *mut PyObject = null_mut();
 pub static mut ARRAY_STRUCT_STR: *mut PyObject = null_mut();
+pub static mut DTYPE_STR: *mut PyObject = null_mut();
+pub static mut DESCR_STR: *mut PyObject = null_mut();
 pub static mut VALUE_STR: *mut PyObject = null_mut();
 pub static mut INT_ATTR_STR: *mut PyObject = null_mut();
 
@@ -145,6 +149,8 @@ pub fn init_typerefs() {
         FIELD_TYPE_STR = PyUnicode_InternFromString("_field_type\0".as_ptr() as *const c_char);
         ARRAY_STRUCT_STR =
             PyUnicode_InternFromString("__array_struct__\0".as_ptr() as *const c_char);
+        DTYPE_STR = PyUnicode_InternFromString("dtype\0".as_ptr() as *const c_char);
+        DESCR_STR = PyUnicode_InternFromString("descr\0".as_ptr() as *const c_char);
         VALUE_STR = PyUnicode_InternFromString("value\0".as_ptr() as *const c_char);
         DEFAULT = PyUnicode_InternFromString("default\0".as_ptr() as *const c_char);
         EXT_HOOK = PyUnicode_InternFromString("ext_hook\0".as_ptr() as *const c_char);
@@ -159,11 +165,9 @@ pub fn init_typerefs() {
 }
 
 #[cold]
-unsafe fn look_up_numpy_type(numpy_module: *mut PyObject, np_type: &str) -> *mut PyTypeObject {
-    let mod_dict = PyObject_GenericGetDict(numpy_module, null_mut());
-    let ptr = PyMapping_GetItemString(mod_dict, np_type.as_ptr() as *const c_char);
+unsafe fn look_up_numpy_type(numpy_module_dict: *mut PyObject, np_type: &str) -> *mut PyTypeObject {
+    let ptr = PyMapping_GetItemString(numpy_module_dict, np_type.as_ptr() as *const c_char);
     Py_XDECREF(ptr);
-    Py_XDECREF(mod_dict);
     ptr as *mut PyTypeObject
 }
 
@@ -176,20 +180,24 @@ pub fn load_numpy_types() -> Box<Option<NonNull<NumpyTypes>>> {
             return Box::new(None);
         }
 
+        let numpy_module_dict = PyObject_GenericGetDict(numpy, null_mut());
         let types = Box::new(NumpyTypes {
-            array: look_up_numpy_type(numpy, "ndarray\0"),
-            float32: look_up_numpy_type(numpy, "float32\0"),
-            float64: look_up_numpy_type(numpy, "float64\0"),
-            int8: look_up_numpy_type(numpy, "int8\0"),
-            int16: look_up_numpy_type(numpy, "int16\0"),
-            int32: look_up_numpy_type(numpy, "int32\0"),
-            int64: look_up_numpy_type(numpy, "int64\0"),
-            uint16: look_up_numpy_type(numpy, "uint16\0"),
-            uint32: look_up_numpy_type(numpy, "uint32\0"),
-            uint64: look_up_numpy_type(numpy, "uint64\0"),
-            uint8: look_up_numpy_type(numpy, "uint8\0"),
-            bool_: look_up_numpy_type(numpy, "bool_\0"),
+            array: look_up_numpy_type(numpy_module_dict, "ndarray\0"),
+            float16: look_up_numpy_type(numpy_module_dict, "half\0"),
+            float32: look_up_numpy_type(numpy_module_dict, "float32\0"),
+            float64: look_up_numpy_type(numpy_module_dict, "float64\0"),
+            int8: look_up_numpy_type(numpy_module_dict, "int8\0"),
+            int16: look_up_numpy_type(numpy_module_dict, "int16\0"),
+            int32: look_up_numpy_type(numpy_module_dict, "int32\0"),
+            int64: look_up_numpy_type(numpy_module_dict, "int64\0"),
+            uint16: look_up_numpy_type(numpy_module_dict, "uint16\0"),
+            uint32: look_up_numpy_type(numpy_module_dict, "uint32\0"),
+            uint64: look_up_numpy_type(numpy_module_dict, "uint64\0"),
+            uint8: look_up_numpy_type(numpy_module_dict, "uint8\0"),
+            bool_: look_up_numpy_type(numpy_module_dict, "bool_\0"),
+            datetime64: look_up_numpy_type(numpy_module_dict, "datetime64\0"),
         });
+        Py_XDECREF(numpy_module_dict);
         Py_XDECREF(numpy);
         Box::new(Some(nonnull!(Box::<NumpyTypes>::into_raw(types))))
     }
